"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncIter = void 0;
const react_1 = require("react");
const useLatest_js_1 = require("../common/hooks/useLatest.js");
const isAsyncIter_js_1 = require("../common/isAsyncIter.js");
const useSimpleRerender_js_1 = require("../common/hooks/useSimpleRerender.js");
const reactAsyncIterSpecialInfoSymbol_js_1 = require("../common/reactAsyncIterSpecialInfoSymbol.js");
// TODO: The initial value can be given as a function, which the internal `useState` would invoke as it's defined to do. So the typings should take into account it possibly being a function and if that's the case then to extract its return type instead of using the function type itself
/**
 * `useAsyncIter` hooks up a single async iterable value into your component and its lifecycle.
 *
 * _Illustration:_
 *
 * ```tsx
 * import { useAsyncIter } from 'react-async-iterators';
 *
 * function SelfUpdatingTodoList(props) {
 *   const { value: todos } = useAsyncIter(props.todosAsyncIter);
 *   return (
 *     <ul>
 *       {todos?.map(todo => (
 *         <li key={todo.id}>{todo.text}</li>
 *       ))}
 *     </ul>
 *   );
 * }
 * ```
 *
 * Given an async iterable `input`, this hook will iterate it and rerender the host component upon
 * each new value that becomes available together with any possible completion or error it may run into.
 * If `input` is a plain (non async iterable) value, it will simply be used to render once and
 * immediately.
 *
 * The hook inits and maintains its current iteration process with its given `input` async iterable
 * across re-renders as long as `input` is passed the same object reference each time (similar to
 * the behavior of a `useEffect(() => {...}, [input])`), therefore care should be taken to avoid
 * constantly recreating the iterable every render, e.g; by declaring it outside the component body,
 * control __when__ it should be recreated with React's
 * [`useMemo`](https://react.dev/reference/react/useMemo) or alternatively the library's
 * {@link iterateFormatted `iterateFormatted`} util for only formatting the values.
 * Whenever `useAsyncIter` detects a different `input` value, it automatically closes a previous
 * `input` async iterable before proceeding to iterate any new `input` async iterable. The hook will
 * also ensure closing a currently iterated `input` on component unmount.
 *
 * The object returned from `useAsyncIter` holds all the state from the most recent iteration
 * of `input` (most recent value, whether is completed or still running, etc. - see
 * {@link IterationResult `IterationResult`}).
 * In case `input` is given a plain value, it will be delivered as-is within the returned
 * result object's `value` property.
 *
 * @template TVal The type of values yielded by the passed iterable or of a plain value passed otherwise.
 * @template TInitVal The type of the initial value, defaults to `undefined`.
 *
 * @param input Any async iterable or plain value
 * @param initialVal Any initial value for the hook to return prior to resolving the ___first
 * emission___ of the ___first given___ async iterable, defaults to `undefined`.
 *
 * @returns An object with properties reflecting the current state of the iterated async iterable
 * or plain value provided via `input` (see {@link IterationResult `IterationResult`})
 *
 * @see {@link IterationResult `IterationResult`}
 *
 * @example
 * ```tsx
 * // With an `initialVal` and showing usage of all properties of the returned iteration object:
 *
 * import { useAsyncIter } from 'react-async-iterators';

 * function SelfUpdatingTodoList(props) {
 *   const todosNext = useAsyncIter(props.todosAsyncIter, []);
 *   return (
 *     <>
 *       {todosNext.error ? (
 *         <div>An error was encountered: {todosNext.error.toString()}</div>
 *       ) : todosNext.done && (
 *         <div>No additional updates for todos are expected</div>
 *       )}
 *
 *       {todosNext.pendingFirst ? (
 *         <div>Loading first todos...</div>
 *       ) : (
 *         <ul>
 *           {todosNext.map(todo => (
 *             <li key={todo.id}>{todo.text}</li>
 *           ))}
 *         </ul>
 *       )}
 *     </>
 *   );
 * }
 * ```
 */
const useAsyncIter = (input, initialVal) => {
    const rerender = (0, useSimpleRerender_js_1.useSimpleRerender)();
    const stateRef = (0, react_1.useRef)({
        value: initialVal,
        pendingFirst: true,
        done: false,
        error: undefined,
    });
    const latestInputRef = (0, useLatest_js_1.useLatest)(input);
    if (!(0, isAsyncIter_js_1.isAsyncIter)(latestInputRef.current)) {
        (0, react_1.useMemo)(() => { }, [undefined]);
        (0, react_1.useEffect)(() => { }, [undefined]);
        stateRef.current = {
            value: latestInputRef.current,
            pendingFirst: false,
            done: false,
            error: undefined,
        };
        return stateRef.current;
    }
    else {
        const iterSourceRefToUse = latestInputRef.current[reactAsyncIterSpecialInfoSymbol_js_1.reactAsyncIterSpecialInfoSymbol]?.origSource ?? latestInputRef.current;
        (0, react_1.useMemo)(() => {
            stateRef.current = {
                value: stateRef.current.value,
                pendingFirst: true,
                done: false,
                error: undefined,
            };
        }, [iterSourceRefToUse]);
        (0, react_1.useEffect)(() => {
            const iterator = iterSourceRefToUse[Symbol.asyncIterator]();
            let iteratorClosedByConsumer = false;
            (async () => {
                let iterationIdx = 0;
                try {
                    for await (const value of { [Symbol.asyncIterator]: () => iterator }) {
                        if (!iteratorClosedByConsumer) {
                            const formattedValue = latestInputRef.current?.[reactAsyncIterSpecialInfoSymbol_js_1.reactAsyncIterSpecialInfoSymbol]?.formatFn(value, iterationIdx++) ?? value;
                            stateRef.current = {
                                value: formattedValue,
                                pendingFirst: false,
                                done: false,
                                error: undefined,
                            };
                            rerender();
                        }
                    }
                    if (!iteratorClosedByConsumer) {
                        stateRef.current = {
                            value: stateRef.current.value,
                            pendingFirst: false,
                            done: true,
                            error: undefined,
                        };
                        rerender();
                    }
                }
                catch (err) {
                    if (!iteratorClosedByConsumer) {
                        stateRef.current = {
                            value: stateRef.current.value,
                            pendingFirst: false,
                            done: true,
                            error: err,
                        };
                        rerender();
                    }
                }
            })();
            return () => {
                iteratorClosedByConsumer = true;
                iterator.return?.();
            };
        }, [iterSourceRefToUse]);
        return stateRef.current;
    }
};
exports.useAsyncIter = useAsyncIter;
//# sourceMappingURL=index.js.map